@@ -1,9 +1,8 @@
-from pprint import pprint
-
 from cloudmesh.abstract.ComputeNodeABC import ComputeNodeABC
 from cloudmesh.common.Printer import Printer
 from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.console import Console
+from cloudmesh.common.debug import VERBOSE
 from cloudmesh.common.dotdict import dotdict
 from cloudmesh.common.parameter import Parameter
 from cloudmesh.common.variables import Variables
@@ -12,9 +11,6 @@ from cloudmesh.management.configuration.name import Name
 from cloudmesh.mongo.CmDatabase import CmDatabase
 from cloudmesh.mongo.DataBaseDecorator import DatabaseUpdate
 from cloudmesh.provider import Provider as ProviderList
-from cloudmesh.common.debug import VERBOSE
-
-from cloudmesh.mongo.CmDatabase import CmDatabase
 
 
 class Provider(ComputeNodeABC):
@@ -129,11 +125,11 @@ class Provider(ComputeNodeABC):
         # bug should determine provider from name
         return self.loop_name(name, self.p.destroy)
 
-    def loop_name(self, names, func):
+    def loop_name(self, names, func, **kwargs):
         names = self.expand(names)
         r = []
         for name in names:
-            vm = func(name=name)
+            vm = func(name=name, **kwargs)
             if type(vm) == list:
                 r = r + vm
             elif type(vm) == dict:
@@ -265,6 +261,17 @@ class Provider(ComputeNodeABC):
 
         return created
 
+    def compress(self, cm):
+        """
+        Opensatck metadata is limited, we remove the spaces
+
+        :param cm:
+        :return:
+        """
+        cm = cm.replace(": ", ":").replace(", ", ":")
+        return cm
+
+
     def _create(self, **arguments):
 
         arguments = dotdict(arguments)
@@ -273,13 +280,18 @@ class Provider(ComputeNodeABC):
 
         StopWatch.start(f"create vm {arguments.name}")
 
+        label = arguments.get("label") or arguments.name
+
         cm = {
             'kind': "vm",
             'name': arguments.name,
+            'label': label,
             'group': arguments.group,
             'cloud': self.cloudname(),
             'status': 'booting'
         }
+
+
         entry = {}
         entry.update(cm=cm, name=arguments.name)
 
@@ -296,12 +308,12 @@ class Provider(ComputeNodeABC):
         # print('entry')
         # pprint(entry)
         # print('data')
-        pprint(data)
+        # pprint(data)
         entry.update(data)
 
         StopWatch.stop(f"create vm {arguments.name}")
         t = format(StopWatch.get(f"create vm {arguments.name}"), '.2f')
-        cm['creation_time'] = t
+        cm['creation'] = t
 
         entry.update({'cm': cm})
 
@@ -313,7 +325,23 @@ class Provider(ComputeNodeABC):
                                            "size": arguments.size})})
 
         cm['status'] = 'available'
-        self.p.set_server_metadata(arguments.name, cm)
+
+        try:
+            #
+            # due to metadata limitation in openstack do not add the creation time
+            #
+
+            if 'created' in cm:
+                del cm['created']
+
+            self.p.set_server_metadata(arguments.name, cm)
+            #self.set_server_metadata(arguments.name, cm)
+        except Exception as e:
+            Console.error("Openstack reported the following error")
+
+            Console.error(79 * "-")
+            print(e)
+            Console.error(79 * "-")
 
         result = CmDatabase.UPDATE(entry, progress=False)[0]
 
@@ -341,7 +369,7 @@ class Provider(ComputeNodeABC):
         return self.loop_name(name, self.p.start)
 
     # @DatabaseUpdate()
-    def info(self, name=None):
+    def info(self, name=None, **kwargs):
         # BUG: needs to work on name and not provider
         return self.loop_name(name, self.p.info)
 
