@@ -1,7 +1,5 @@
 from cloudmesh.common.console import Console
-from pprint import pprint
 from cloudmesh.common.variables import Variables
-from cloudmesh.common.debug import VERBOSE
 
 
 class ComputeProviderPlugin(object):
